var listGrid = isc.ListGrid.create({
    width:900,
    height:500,
    dataSource:window.employees,
    autoFetchData:true,
    canEdit:true,
    canGroupBy:false,
    canReorderRecords:true,
    canSort:false, // Disable user sorting because we rely on records being sorted by 'userOrder'.
    autoSaveEdits:true,

    recordDrop : function (dropRecords, targetRecord, targetIndex, sourceWidget) {
        if (this == sourceWidget && dropRecords.length != 0) {
            var data = this.data;
            var dropRecordIndices = dropRecords.map(function (record) {
                return data.findIndex("EmployeeId", record.EmployeeId);
            });

            var indicesMin = Math.min(dropRecordIndices.min(), targetIndex);
            var indicesMax = Math.max(dropRecordIndices.max(), targetIndex - 1);

            var startedQueue = !isc.RPCManager.startQueue();
            var ds = isc.DS.get(this.dataSource);
            var request = {
                operation:this.updateOperation,
                application:this.application,
                willHandleError:true,
                oldValues:{ userOrder:0 },
                componentId:this.ID
            };

            // Update the 'userOrder' fields for all records at indices [indicesMin, indicesMax].
            var userOrders = new Array(indicesMax + 1 - indicesMin);
            var i;
            for (i = indicesMin; i <= indicesMax; ++i) {
                var record = data.get(i);
                userOrders[i - indicesMin] = record.userOrder;
            }
            var numDropRecordsAfterOrAtTargetIndex = 0;
            for (i = indicesMax; i >= targetIndex; --i) {
                if (dropRecordIndices.contains(i)) {
                    ++numDropRecordsAfterOrAtTargetIndex;
                }
            }
            var numDropRecordsBeforeTargetIndex = dropRecords.length - numDropRecordsAfterOrAtTargetIndex;
            var j = 0; // how many drop records have been encountered so far.
            for (i = indicesMin; i < targetIndex; ++i) {
                var record = data.get(i);
                request.oldValues.userOrder = record.userOrder;
                request._originalRecord = isc.shallowClone(record);

                var updates = ds.filterPrimaryKeyFields(record);
                if (dropRecordIndices.contains(i)) {
                    updates.userOrder = record.userOrder = userOrders[targetIndex - numDropRecordsBeforeTargetIndex + j - indicesMin];
                    ++j;
                } else {
                    updates.userOrder = record.userOrder = userOrders[i - j - indicesMin];
                }
                ds.updateData(updates, null, request);
            }
            j = 0;
            for (i = indicesMax; i >= targetIndex; --i) {
                var record = data.get(i);
                request.oldValues.userOrder = record.userOrder;
                request._originalRecord = isc.shallowClone(record);

                var updates = ds.filterPrimaryKeyFields(record);
                if (dropRecordIndices.contains(i)) {
                    updates.userOrder = record.userOrder = userOrders[targetIndex + numDropRecordsAfterOrAtTargetIndex - 1 - j - indicesMin];
                    ++j;
                } else {
                    updates.userOrder = record.userOrder = userOrders[i + j - indicesMin];
                }
                ds.updateData(updates, null, request);
            }

            // If we're queuing, send the queue now.
            if (startedQueue) {
                isc.RPCManager.sendQueue(null, null, null, true);
            }
        }

        // Call the super implementation of recordDrop() to update the order of rows in the ListGrid.
        this.Super("recordDrop", arguments);
    }
});
listGrid.addSort({ property:"userOrder" });